/*
 * SPDX-FileCopyrightText: (C) 2025 Siemens
 * SPDX-License-Identifier: LGPL-2.1-only
 */

#pragma once

#if !defined(_SSO_MIB_INSIDE_) && !defined(SSO_MIB_COMPILATION)
#error "Only <sso-mib.h> can be included directly."
#endif

#ifdef BUILDING_SSO_MIB
#define PUBLIC_API __attribute__((visibility("default")))
#else
#define PUBLIC_API
#endif
